# TST - work time tracker
# tst10/db.py: classes for database access
# Copyright (C) 2005 The TST Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

from MySQLdb import connect
import datetime

prefix = __name__.split(".")[0]
prefix_end = len(prefix) + 1
task_table = "%s_task" % prefix
session_table = "%s_session" % prefix

class Error(Exception):
    pass

def fix(s):
    # Prevent SQL injection attacks.
    return s.replace("\\", "\\\\").replace("'", "\\'")

def check_user(s):
    if len(s) == 0:
        msg = _("User name cannot be left blank") + "."
        raise Error, msg
    for c in s:
        if not c.isalnum():
            msg = _("Invalid character in user name")
            raise Error, "%s. (%s)" % (msg, repr(c))
    return s

def check_task(s):
    if len(s) == 0:
        msg = _("Task name cannot be left blank") + "."
        raise Error, msg
    return s

class DB:
    def __init__(self):
        self.db = None
    def connected(self):
        return self.db != None and self.db.open
    def connect(self, db, user, passwd, host, port):
        if self.connected():
            self.db.close()
        try:
            self.db = connect(
                db = db, user = user, passwd = passwd, host = host,
                port = port)
            self.db.autocommit(True)
        except Exception, e:
            if e.args[0] in (1044, 1045, 1049, 1130):
                msg = _("Incorrect user or password")
            elif e.args[0] in (2002, 2003, 2005):
                msg = _("Cannot connect to server")
            else:
                msg = e.args[1]
            raise Error, "%s. (%d)" % (msg, e.args[0])
    def query(self, query):
        if type(query) is list:
            for item in query:
                self.query(item)
        else:
            try:
                self.db.query(query)
            except Exception, e:
                # When connection with MySQL server is lost, MySQLdb
                # returns error code 0.  In the next try returns a
                # more helpful code.
                if e.args[0] == 0:
                    self.query(query)
                    return
                if e.args[0] in (0, 2003, 2006, 2013):
                    msg = _("Connection with server has been lost")
                elif e.args[0] in (1007, 1062):
                    msg = _("This entry already exists")
                else:
                    msg = str(e.args[1])
                raise Error, "%s. (%d)" % (msg, e.args[0])
    def disconnect(self):
        self.db.close()

class AdminDB(DB):
    def connect(self, user, passwd, host, port):
        DB.connect(self, "mysql", user, passwd, host, port)
    def users(self):
        query = "SELECT DISTINCT SUBSTRING(`User` FROM %d + 1) AS `User` " \
        	"FROM `user` WHERE LEFT(`User`, %d) = '%s_' " \
                "ORDER BY `User`" \
                % (prefix_end, prefix_end, prefix)
        self.query(query)
        result = self.db.store_result()
        users = []
        while True:
            row = result.fetch_row()
            if not row:
                break
            if len(row[0][0]) > 0:
                users.append(row[0][0])
        return users
    def add_user(self, user, password):
        user = "%s_%s" % (prefix, check_user(user))
        password = fix(password)
        queries = [
            "CREATE DATABASE `%s`" % (user),
            "CREATE TABLE `%s`.`%s_task` (" \
            "`task_id` INT NOT NULL AUTO_INCREMENT," \
            "`task_name` VARCHAR(32) NOT NULL," \
            "`task_notes` TEXT NOT NULL," \
            "PRIMARY KEY (`task_id`)," \
            "UNIQUE (`task_name`))" % (user, prefix),
            "CREATE TABLE `%s`.`%s_session` (" \
            "`session_id` INT NOT NULL AUTO_INCREMENT," \
            "`session_task_id` INT NOT NULL," \
            "`session_detail` TEXT NOT NULL," \
            "`session_date` DATE NOT NULL," \
            "`session_time` TIME NOT NULL," \
            "PRIMARY KEY (`session_id`)," \
            "INDEX (`session_task_id`))" % (user, prefix),
            "GRANT USAGE ON *.* TO `%s`@`localhost` " \
            "IDENTIFIED BY '%s'" % (user, password),
            "GRANT USAGE ON *.* TO `%s`@`%%` " \
            "IDENTIFIED BY '%s'" % (user, password),
            "GRANT SELECT, INSERT, UPDATE, DELETE ON `%s`.* " \
            "TO `%s`@`localhost`" % (user, user),
            "GRANT SELECT, INSERT, UPDATE, DELETE ON `%s`.* " \
            "TO `%s`@`%%`" % (user, user),
            "FLUSH PRIVILEGES"]
        self.query(queries)
    def set_user_password(self, user, password):
        user = "%s_%s" % (prefix, user)
        password = fix(password)
        queries = [
            "SET PASSWORD FOR `%s`@`localhost` = " \
            "PASSWORD('%s')" % (user, password),
            "SET PASSWORD FOR `%s`@`%%` = " \
            "PASSWORD('%s')" % (user, password),
            "FLUSH PRIVILEGES"]
        self.query(queries)
    def del_user(self, user):
        user = "%s_%s" % (prefix, user)
        user_fixed = fix(user)
        queries = [
            "REVOKE ALL PRIVILEGES ON *.* " \
            "FROM `%s`@`localhost`" % (user),
            "DELETE FROM `user` WHERE `User` = '%s' " \
            "AND `Host` = 'localhost'" % (user_fixed),
            "REVOKE ALL PRIVILEGES ON *.* " \
            "FROM `%s`@`%%`" % (user),
            "DELETE FROM `user` WHERE `User` = '%s' " \
            "AND `Host` = '%%'" % (user_fixed),
            "DELETE FROM `db` WHERE `Host` = 'localhost' " \
            "AND `Db` = '%s' AND `User` = '%s'" % (user_fixed, user_fixed),
            "DELETE FROM `db` WHERE `Host` = '%%' " \
            "AND `Db` = '%s' AND `User` = '%s'" % (user_fixed, user_fixed),
            "DROP DATABASE `%s`" % (user)]
        self.query(queries)

class UserDB(DB):
    def connect(self, user, passwd, host, port):
        user = "%s_%s" % (prefix, user)
        DB.connect(self, user, user, passwd, host, port)
    def tasks(self):
        query = "SELECT * FROM `%s` ORDER BY `task_name`" % (task_table)
        self.query(query)
        result = self.db.store_result()
        tasks = []
        while True:
            row = result.fetch_row()
            if not row:
                break
            task = {}
            task["id"] = row[0][0]
            task["name"] = row[0][1]
            task["notes"] = row[0][2]
            tasks.append(task)
        return tasks
    def add_task(self, name, notes):
        name = check_task(fix(name))
        notes = fix(notes)
        query = "INSERT INTO `%s` VALUES ('%%', '%s', '%s')" \
                % (task_table, name, notes)
        self.query(query)
    def del_task(self, id):
        queries = [
            "DELETE FROM `%s` WHERE `task_id` = '%d'" % (task_table, id),
            "DELETE FROM `%s` WHERE `session_task_id` = '%d'" \
            % (session_table, id)]
        self.query(queries)
    def mod_task(self, id, name, notes):
        name = check_task(fix(name))
        notes = fix(notes)
        query = "UPDATE `%s` SET `task_name` = '%s', `task_notes` = '%s' " \
                "WHERE `task_id` = '%d'" % (task_table, name, notes, id)
        self.query(query)
    def sessions(self, task_id = None, date1 = None, date2 = None):
        query = "SELECT * FROM `%s`" % (session_table)
        conds = []
        if task_id != None:
            conds.append("`session_task_id` = '%d'" % task_id)
        if date1 != None:
            conds.append("`session_date` >= '%s'" % str(date1))
        if date2 != None:
            conds.append("`session_date` <= '%s'" % str(date2))
        conds = " AND ".join(conds)
        if len(conds) > 0:
            query += " WHERE " + conds
        query += " ORDER BY `session_date`, `session_id`" 
        self.query(query)
        result = self.db.store_result()
        sessions = []
        while True:
            row = result.fetch_row()
            if not row:
                break
            session = {}
            session["id"] = row[0][0]
            session["task_id"] = row[0][1]
            session["detail"] = row[0][2]
            session["date"] = row[0][3]
            # Some versions of MySQLdb return a datetime.timedelta
            # instead of a datetime.datetime.
            if type(row[0][4]) is datetime.timedelta:
                ss = row[0][4].seconds
                mm = ss / 60
                ss = ss % 60
                hh = mm / 60
                mm = mm % 60
                session["time"] = datetime.time(hh, mm, ss)
            else:
                session["time"] = row[0][4]
            sessions.append(session)
        return sessions
    def add_session(self, task_id, detail, date, time):
        detail = fix(detail)
        query = "INSERT INTO `%s` VALUES ('%%', '%d', '%s', '%s', '%s')" \
                % (session_table, task_id, detail, str(date), str(time))
        self.query(query)
    def del_session(self, id):
        query = "DELETE FROM `%s` WHERE `session_id` = '%d'" \
                % (session_table, id)
        self.query(query)
    def mod_session(self, id, detail, date, time):
        detail = fix(detail)
        query = "UPDATE `%s` SET `session_detail` = '%s', " \
                "`session_date` = '%s', `session_time` = '%s' " \
                "WHERE `session_id` = '%d'" \
                % (session_table, detail, str(date), str(time), id)
        self.query(query)
    def sum_sessions(self, task_id = None, date1 = None, date2 = None):
        query = "SELECT SUM(TIME_TO_SEC(`session_time`)) " \
                "FROM `%s`" % (session_table)
        conds = []
        if task_id != None:
            conds.append("`session_task_id` = '%d'" % task_id)
        if date1 != None:
            conds.append("`session_date` >= '%s'" % str(date1))
        if date2 != None:
            conds.append("`session_date` <= '%s'" % str(date2))
        conds = " AND ".join(conds)
        if len(conds) > 0:
            query += " WHERE " + conds
        self.query(query)
        result = self.db.store_result()
        row = result.fetch_row()
        if row and row[0][0]:
            return int(row[0][0])
        else:
            return 0
