# TST - work time tracker
# tst10/gui.py: graphical user interface classes
# Copyright (C) 2005 The TST Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

import gtk
import gobject
import time
import datetime
import os
import sys

from tst10 import inst_prefix

prefix = __name__.split(".")[0]
user_name_len = 16 - len(prefix) - 1

def begin():
    gtk.main()

def end():
    gtk.main_quit()

def get_image_path(image):
    return os.path.join(
        inst_prefix, "share", "pixmaps", "tst10", image)

class GripButton(gtk.EventBox):
    def __init__(self):
        gtk.EventBox.__init__(self)
        self.x, self.y = 0, 0
        self.pressed = False
        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_OUT)
        vseparator = gtk.VSeparator()
        vseparator.set_size_request(5, -1)
        frame.add(vseparator)
        self.add(frame)
        self.connect("button-press-event", self.on_button_press)
        self.connect("motion-notify-event", self.on_motion_notify)
        self.connect("button-release-event", self.on_button_release)
        self.connect("enter-notify-event", self.on_enter_notify)
        self.connect("leave-notify-event", self.on_leave_notify)
        self.move_cursor = gtk.gdk.Cursor(gtk.gdk.FLEUR)
    def on_button_press(self, event_box, event):
        self.x, self.y = event.x, event.y
        self.pressed = True
        return True
    def on_motion_notify(self, event_box, event):
        widget = self.get_toplevel()
        if type(widget) is gtk.Window:
            x, y = widget.get_position()
            x += int(event.x - self.x)
            y += int(event.y - self.y)
            screen = widget.get_screen()
            mx, my = widget.get_size()
            mx = screen.get_width() - mx
            my = screen.get_height() - my
            if x < 0:
                x = 0
            elif x > mx:
                x = mx
            if y < 0:
                y = 0
            elif y > my:
                y = my
            widget.move(x, y)
        return True
    def on_button_release(self, event_box, event):
        self.pressed = False
        return True
    def on_enter_notify(self, event_box, event):
        widget = self.get_toplevel()
        if type(widget) is gtk.Window:
            widget.window.set_cursor(self.move_cursor)
        return True
    def on_leave_notify(self, event_box, event):
        if not self.pressed:
            widget = self.get_toplevel()
            if type(widget) is gtk.Window:
                widget.window.set_cursor(None)
        return True

class ImageLabelToggleButton(gtk.ToggleButton):
    def __init__(self, stock_image, label):
        gtk.ToggleButton.__init__(self)
        image = gtk.Image()
        image.set_from_stock(stock_image, gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.label = gtk.Label(label)
        hbox = gtk.HBox()
        hbox.set_spacing(2)
        hbox.pack_start(image, False, False)
        hbox.pack_start(self.label, False, False)
        self.add(hbox)
        self.bstyle = self.rc_get_style()
        self.lstyle = self.label.rc_get_style()
        self.timeout = None
        self.normal = False
    def set_label(self, label):
        self.label.set_label(label)
    def set_alert(self, alert):
        if alert:
            self.normal = False
            self.timeout = gobject.timeout_add(10000, self.alert_start)
        elif self.timeout is not None:
            gobject.source_remove(self.timeout)
            if self.normal:
                self.alert_update()
            self.timeout = None
    def alert_start(self):
        self.timeout = gobject.timeout_add(1000, self.alert_update)
        return False
    def alert_update(self):
        states = (gtk.STATE_NORMAL, gtk.STATE_PRELIGHT)
        if self.normal:
            for state in states:
                self.modify_bg(state, self.bstyle.bg[state])
                self.label.modify_fg(state, self.lstyle.fg[gtk.STATE_NORMAL])
        else:
            for state in states:
                self.modify_bg(state, self.lstyle.bg[gtk.STATE_SELECTED])
                self.label.modify_fg(state, self.lstyle.fg[gtk.STATE_SELECTED])
        self.normal = not self.normal
        return True

class ImageButton(gtk.Button):
    def __init__(self, stock_image):
        gtk.Button.__init__(self)
        image = gtk.Image()
        image.set_from_stock(stock_image, gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.add(image)

def seconds_to_time(seconds):
    return datetime.time(seconds / 3600, seconds % 3600 / 60, seconds % 60)

class WorkWindow:
    def __init__(self, conf):
        # In GTK+ 2.4 and above we can use the method set_keep_above
        # to keep the window above the others.  A workaround for older
        # versions is to set the window type to WINDOW_POPUP.
        if gtk.gtk_version >= (2, 4, 0):
            self.window = gtk.Window()
            self.window.set_resizable(False)
            self.window.set_decorated(False)
        else:
            self.window = gtk.Window(gtk.WINDOW_POPUP)
        self.window.connect("delete-event", lambda w, e: True)
        hbox = gtk.HBox()
        self.window.add(hbox)
        hbox.pack_start(GripButton(), False, False)
        self.work_button = ImageLabelToggleButton(gtk.STOCK_EXECUTE, '')
        self.work_button.connect("toggled", self.on_work_button_toggled)
        hbox.pack_start(self.work_button, False, False)
        self.stop_button = ImageButton(gtk.STOCK_STOP)
        hbox.pack_start(self.stop_button, False, False)
        self.tooltips = gtk.Tooltips()
        self.tooltips.set_tip(self.stop_button, _("End work session"))
        # This is a fix for what seems to be a bug in GTK+ for
        # Windows.  If not done this way the window is displayed
        # incorrectly.
        if sys.platform[:3] == "win":
            self.window.show_all()
            self.window.hide()
        else:
            hbox.show_all()
        try:
            self.x = int(conf["work_window_x"])
            self.y = int(conf["work_window_y"])
        except:
            screen = self.window.get_screen()
            self.x = screen.get_width() - self.window.get_size()[0] - 75
            self.y = 0
    def set_stop_callback(self, function):
        self.stop_button.connect("clicked", self.on_stop_button_clicked)
        self.stop_callback = function
    def start(self, task, notes):
        self.window.set_title(_("Work session"))
        tip = "%s" % task
        if len(notes) > 0:
            tip += "\n\n" + _("Notes") + ":\n%s" % notes
        self.tooltips.set_tip(self.work_button, tip)
        self.total_time = 0
        self.work_button.set_alert(False)
        self.work_button.set_label('00:00:00')
        self.work_button.set_active(True)
        self.window.move(self.x, self.y)
        self.window.show()
        if gtk.gtk_version >= (2, 4, 0):
            self.window.set_keep_above(True)
            self.window.stick()
    def on_work_button_toggled(self, work_button):
        if work_button.get_active():
            self.work_button.set_alert(False)
            self.t = time.time()
            self.stop_button.set_sensitive(False)
            self.timeout = gobject.timeout_add(500, self.update)
        else:
            self.work_button.set_alert(True)
            self.stop_button.set_sensitive(True)
            gobject.source_remove(self.timeout)
            self.total_time = self.time
        return True
    def on_stop_button_clicked(self, stop_button):
        self.stop_callback(seconds_to_time(self.total_time))
        if gtk.gtk_version >= (2, 4, 0):
            self.window.set_keep_above(False)
            self.window.unstick()
        self.x, self.y = self.window.get_position()
        self.window.hide()
        return True
    def update(self):
        s = int(time.time() - self.t)
        if s < 0:
            self.work_button.set_active(False)
        else:
            self.time = self.total_time + s
            if self.time >= 86400:
                self.time = 86399
                self.work_button.set_active(False)
                self.stop_button.clicked()
            self.work_button.set_label(str(seconds_to_time(self.time)))
        return True
    def save_conf(self, conf):
        conf["work_window_x"] = str(self.x)
        conf["work_window_y"] = str(self.y)

class Message(gtk.EventBox):
    def __init__(self):
        gtk.EventBox.__init__(self)
        hbox = gtk.HBox()
        hbox.set_spacing(5)
        self.add(hbox)
        self.image = gtk.Image()
        self.image.set_from_stock(
            gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_SMALL_TOOLBAR)
        hbox.pack_start(self.image, False, False)
        self.entry = gtk.Entry()
        self.entry.set_editable(False)
        hbox.pack_start(self.entry)
        self.tip = _("There was an error") + "..."
        self.tooltips = gtk.Tooltips()
        self.tooltips.set_tip(self, self.tip)
        self.sensitive = True
        self.alert_count = 0
        self.timeout = None
    def set_alert(self, alert):
        if alert:
            if self.alert_count > 0:
                self.alert_count = 0
            else:
                self.timeout = gobject.timeout_add(125, self.alert_update)
        elif self.timeout is not None:
            self.alert_count = 0
            gobject.source_remove(self.timeout)
            self.image.set_sensitive(self.sensitive)
            self.timeout = None
    def set_message(self, message):
        self.entry.set_text(message)
    def set_sensitive(self, sensitive):
        self.sensitive = sensitive
        self.image.set_sensitive(sensitive)
        self.entry.set_sensitive(sensitive)
        if sensitive:
            self.tooltips.set_tip(self, self.tip)
        else:
            self.tooltips.set_tip(self, None)
    def alert_update(self):
        self.alert_count += 1
        if self.alert_count == 7:
            self.alert_count = 0
            self.image.set_sensitive(self.sensitive)
            self.timeout = None
            return False
        if self.alert_count & 1:
            self.image.set_sensitive(not self.sensitive)
        else:
            self.image.set_sensitive(self.sensitive)
        return True

class BasicWindow:
    def __init__(self):
        self.vbox = gtk.VBox()
        self.vbox.set_border_width(10)
        self.vbox.set_spacing(10)
        self.message = Message()
        self.vbox.pack_end(self.message, False, False)
        self.window = gtk.Window()
        self.window.add(self.vbox)
        self.window.connect("key-press-event", self.on_window_key_press_event)
        self.tooltips = gtk.Tooltips()
        self.working_cursor = gtk.gdk.Cursor(gtk.gdk.WATCH)
    def show(self):
        self.set_message()
        self.window.show_now()
    def hide(self):
        self.window.hide()
        parent = self.window.get_transient_for()
        if not parent is None:
            parent.present()
    def set_working(self, working):
        if self.window.window is not None:
            if working:
                self.window.window.set_cursor(self.working_cursor)
                while gtk.events_pending():
                    gtk.main_iteration()
            else:
                self.window.window.set_cursor(None)
    def set_message(self, message = None):
        if message is None:
            self.message.set_alert(False)
            self.message.set_sensitive(False)
            self.message.set_message("")
        else:
            self.message.set_message(message)
            self.message.set_sensitive(True)
            self.message.set_alert(True)
            gtk.gdk.beep()
    def set_close_callback(self, function):
        self.window.connect("delete-event", self.on_window_delete_event)
        self.close_callback = function
    def on_window_delete_event(self, widget, event):
        self.close_callback()
        return True
    def on_window_key_press_event(self, widget, event):
        if gtk.gdk.keyval_name(event.keyval) == "Escape":
            self.close_callback()
            return True
        return False

class LeftLabel(gtk.Label):
    def __init__(self, label):
        gtk.Label.__init__(self, label)
        self.set_alignment(.0, .5)

class ImageLabelButton(gtk.Button):
    def __init__(self, stock_image, label):
        gtk.Button.__init__(self)
        self.set_relief(gtk.RELIEF_NONE)
        image = gtk.Image()
        image.set_from_stock(stock_image, gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.label = gtk.Label(label)
        hbox = gtk.HBox()
        hbox.set_spacing(2)
        hbox.pack_start(image, False, False)
        hbox.pack_start(self.label, False, False)
        alignment = gtk.Alignment()
        alignment.set(.5, .5, .0, .0)
        alignment.add(hbox)
        self.add(alignment)
    def set_label(self, label):
        self.label.set_label(label)

class Calendar(gtk.Calendar):
    def __init__(self):
        gtk.Calendar.__init__(self)
        self.connect("month-changed", self.on_month_changed)
    def set_date(self, date):
        self.freeze()
        # Fix to avoid warning.
        self.select_day(1)
        if date.year < 1900:
            self.select_month(date.month - 1, 1900)
        else:
            self.select_month(date.month - 1, date.year)
        self.select_day(date.day)
        self.thaw()
    def get_date(self):
        year, month, day = gtk.Calendar.get_date(self)
        if year < 1900:
            year = 1900
        return datetime.date(year, month + 1, day)
    def on_month_changed(self, calendar):
        date = gtk.Calendar.get_date(self)
        if date[0] < 1900:
            gtk.gdk.beep()
            self.select_month(date[1], 1900)

class ReportWindow(BasicWindow):
    def __init__(self, conf, parent):
        BasicWindow.__init__(self)
        self.window.set_title(_("Report"))
        self.window.set_modal(True)
        self.window.set_transient_for(parent.window)
        try:
            width = int(conf["report_window_width"])
            height = int(conf["report_window_height"])
            self.window.set_default_size(width, height)
        except:
            self.window.set_default_size(600, 440)
        hbox = gtk.HBox()
        hbox.set_spacing(5)
        self.vbox.pack_start(hbox)
        alignment = gtk.Alignment()
        hbox.pack_start(alignment, False, False)
        vbox = gtk.VBox()
        vbox.set_spacing(5)
        alignment.add(vbox)
        vbox.pack_start(LeftLabel(_("From day") + ":"))
        self.calendar1 = Calendar()
        vbox.pack_start(self.calendar1)
        vbox.pack_start(LeftLabel(_("To day") + ":"))
        self.calendar2 = Calendar()
        vbox.pack_start(self.calendar2)
        vbox = gtk.VBox()
        vbox.set_spacing(5)
        hbox.pack_start(vbox)
        vbox.pack_start(LeftLabel(_("Report") + ":"), False, False)
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        vbox.pack_start(scrolled_window)
        self.report_textview = gtk.TextView()
        self.report_textview.set_wrap_mode(gtk.WRAP_NONE)
        self.report_textview.set_editable(False)
        scrolled_window.add(self.report_textview)
        self.copy_button = ImageLabelButton(gtk.STOCK_COPY, _("Copy"))
        self.vbox.pack_start(self.copy_button, False, False)
        self.tooltips.set_tip(
            self.copy_button, _("Copy report to clipboard"))
        self.vbox.show_all()
        self.clipboard = gtk.Clipboard()
        self.primary_clipboard = gtk.Clipboard(selection="PRIMARY")
    def show(self, filter):
        self.copy_button.grab_focus()
        self.calendar1.set_date(filter[0])
        self.calendar2.set_date(filter[1])
        BasicWindow.show(self)
        self.update_callback(
            (self.calendar1.get_date(), self.calendar2.get_date()))
    def set_update_callback(self, function):
        self.calendar1.connect("day-selected", self.on_calendar_day_selected)
        self.calendar2.connect("day-selected", self.on_calendar_day_selected)
        self.update_callback = function
    def set_copy_callback(self, function):
        self.copy_button.connect("clicked", self.on_copy_button_clicked)
        self.copy_callback = function
    def on_calendar_day_selected(self, calendar):
        if self.window.get_property("visible"):
            self.update_callback(
                (self.calendar1.get_date(), self.calendar2.get_date()))
    def on_copy_button_clicked(self, button):
        buffer = self.report_textview.get_buffer()
        start, end = buffer.get_bounds()
	text = buffer.get_text(start, end)
        self.clipboard.set_text(text)
        self.primary_clipboard.set_text(text)
        self.copy_callback()
    def update(self, report):
        self.report_textview.get_buffer().set_text(report)
    def save_conf(self, conf):
        width, height = self.window.get_size()
        conf["report_window_width"] = str(width)
        conf["report_window_height"] = str(height)

class SessionEditWindow(BasicWindow):
    def __init__(self, conf, parent):
        BasicWindow.__init__(self)
        self.window.set_modal(True)
        self.window.set_transient_for(parent.window)
        try:
            width = int(conf["session_edit_window_width"])
            height = int(conf["session_edit_window_height"])
            self.window.set_default_size(width, height)
        except:
            self.window.set_default_size(480, 320)
        table = gtk.Table(2, 2)
        self.vbox.pack_start(table)
        table.set_row_spacings(5)
        table.set_col_spacings(5)
        table.attach(LeftLabel(_("Detail") + ":"), 0, 1, 0, 1, gtk.FILL, 0)
        table.attach(LeftLabel(_("Date") + ":"), 1, 2, 0, 1, gtk.FILL, 0)
        self.detail_textview = gtk.TextView()
        self.detail_textview.set_wrap_mode(gtk.WRAP_WORD)
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.add(self.detail_textview)
        table.attach(
            scrolled_window, 0, 1, 1, 2, gtk.EXPAND | gtk.FILL,
            gtk.EXPAND | gtk.FILL)
        vbox = gtk.VBox()
        table.attach(vbox, 1, 2, 1, 2, gtk.FILL, gtk.FILL)
        vbox.set_spacing(5)
        self.calendar = Calendar()
        vbox.pack_start(self.calendar, False, False)
        vbox.pack_start(LeftLabel(_("Time") + ":"), False, False)
        hbox = gtk.HBox()
        hbox.set_spacing(2)
        vbox.pack_start(hbox, False, False)
        self.hh = gtk.SpinButton(gtk.Adjustment(0, 0, 23, 1))
        self.hh.set_update_policy(gtk.UPDATE_IF_VALID)
        hbox.pack_start(self.hh, False, False)
        self.mm = gtk.SpinButton(gtk.Adjustment(0, 0, 59, 1))
        self.mm.set_update_policy(gtk.UPDATE_IF_VALID)
        hbox.pack_start(self.mm, False, False)
        self.ss = gtk.SpinButton(gtk.Adjustment(0, 0, 59, 1))
        self.ss.set_update_policy(gtk.UPDATE_IF_VALID)
        hbox.pack_start(self.ss, False, False)
        self.save_button = ImageLabelButton(gtk.STOCK_OK, _("Save"))
        self.vbox.pack_start(self.save_button, False, False)
        self.tooltips.set_tip(self.hh, _("Hours"))
        self.tooltips.set_tip(self.mm, _("Minutes"))
        self.tooltips.set_tip(self.ss, _("Seconds"))
        self.tooltips.set_tip(
            self.save_button, _("Save changes made to work session"))
        self.vbox.show_all()
    def show(self, task, session, new = False):
        self.detail_textview.get_buffer().set_text(session["detail"])
        self.calendar.set_date(session["date"])
        self.hh.set_value(session["time"].hour)
        self.mm.set_value(session["time"].minute)
        self.ss.set_value(session["time"].second)
        self.detail_textview.grab_focus()
        if new:
            self.window.set_title(_("New work session"))
        else:
            self.window.set_title(_("Edit work session"))
        BasicWindow.show(self)
    def set_save_callback(self, function):
        self.save_button.connect("clicked", self.on_save_button_clicked)
        self.save_callback = function
    def on_save_button_clicked(self, save_button):
        textbuffer = self.detail_textview.get_buffer()
        start, end = textbuffer.get_bounds()
        data = {}
        data["detail"] = textbuffer.get_text(start, end)
        data["date"] = self.calendar.get_date()
        data["time"] = datetime.time(
            int(self.hh.get_value()), int(self.mm.get_value()),
            int(self.ss.get_value()))
        self.save_callback(data)
        return True
    def save_conf(self, conf):
        width, height = self.window.get_size()
        conf["session_edit_window_width"] = str(width)
        conf["session_edit_window_height"] = str(height)

class Entry(gtk.Entry):
    def __init__(self):
        gtk.Entry.__init__(self)
        self.connect("insert-text", self.on_insert_text)
    def on_insert_text(self, entry, text, length, pos):
        # Fix to handle max_length correcty with UTF-8 strings.
        max_length = self.get_max_length()
        if max_length == 0:
            return False
        if len(self.get_text()) + length > max_length:
            self.stop_emission("insert-text")
            gtk.gdk.beep()
            return True
        return False

class TaskEditWindow(BasicWindow):
    def __init__(self, conf, parent, new = False):
        BasicWindow.__init__(self)
        self.window.set_modal(True)
        self.window.set_transient_for(parent.window)
        try:
            if new:
                width = int(conf["task_new_window_width"])
                height = int(conf["task_new_window_height"])
            else:
                width = int(conf["task_edit_window_width"])
                height = int(conf["task_edit_window_height"])
            self.window.set_default_size(width, height)
        except:
            self.window.set_default_size(320, 200)
        vbox = gtk.VBox()
        self.vbox.pack_start(vbox)
        vbox.set_spacing(5)
        hbox = gtk.HBox()
        vbox.pack_start(hbox, False, False)
        hbox.set_spacing(5)
        hbox.pack_start(LeftLabel(_("Task name") + ":"), False, False)
        self.name_entry = Entry()
        self.name_entry.set_max_length(32)
        self.name_entry.set_activates_default(True)
        hbox.pack_start(self.name_entry)
        self.notes_textview = gtk.TextView()
        self.notes_textview.set_wrap_mode(gtk.WRAP_WORD)
        vbox.pack_start(LeftLabel(_("Notes") + ":"), False, False)
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.add(self.notes_textview)
        vbox.pack_start(scrolled_window)
        self.scrolled_window = scrolled_window
        self.save_button = ImageLabelButton(gtk.STOCK_OK, _("Save"))
        self.vbox.pack_start(self.save_button, False, False)
        self.save_button.set_flags(gtk.CAN_DEFAULT)
        self.save_button.grab_default()
        if new:
            self.window.set_title(_("New task"))
            self.tooltips.set_tip(
                self.name_entry, _("Name that will indentify the new task"))
            self.tooltips.set_tip(self.save_button, _("Save new task"))
        else:
            self.window.set_title(_("Edit task"))
            self.tooltips.set_tip(
                self.name_entry, _("Name of the task that is being edited"))
            self.tooltips.set_tip(
                self.save_button, _("Save changes made to task"))
        self.vbox.show_all()
        self.new = new
    def show(self, data = None):
        if not data is None:
            self.name_entry.set_text(data["name"])
            self.notes_textview.get_buffer().set_text(data["notes"])
            self.notes_textview.grab_focus()
        else:
            self.name_entry.grab_focus()
        BasicWindow.show(self)
    def hide(self):
        self.name_entry.set_text("")
        self.notes_textview.get_buffer().set_text("")
        BasicWindow.hide(self)
    def set_save_callback(self, function):
        self.save_button.connect("clicked", self.on_save_button_clicked)
        self.save_callback = function
    def on_save_button_clicked(self, widget):
        textbuffer = self.notes_textview.get_buffer()
        start, end = textbuffer.get_bounds()
        data = {"name": self.name_entry.get_text(),
                "notes": textbuffer.get_text(start, end)}
        self.save_callback(data)
        return True
    def save_conf(self, conf):
        width, height = self.window.get_size()
        if self.new:
            conf["task_new_window_width"] = str(width)
            conf["task_new_window_height"] = str(height)
        else:
            conf["task_edit_window_width"] = str(width)
            conf["task_edit_window_height"] = str(height)

class ImageLabelMenuItem(gtk.MenuItem):
    def __init__(self, stock_image, label):
        gtk.MenuItem.__init__(self)
        hbox = gtk.HBox()
        hbox.set_spacing(5)
        self.add(hbox)
        image = gtk.Image()
        image.set_from_stock(stock_image, gtk.ICON_SIZE_SMALL_TOOLBAR)
        hbox.pack_start(image, False, False)
        hbox.pack_start(gtk.Label(label), False, False)
        self.show_all()

class EraseMenu(gtk.Menu):
    def __init__(self):
        gtk.Menu.__init__(self)
        self.erase_menu_item = ImageLabelMenuItem(
            gtk.STOCK_REMOVE, _("Delete"))
        self.add(self.erase_menu_item)
    def set_erase_callback(self, function):
        self.erase_menu_item.connect(
            "activate", self.on_erase_menu_item_activated)
        self.erase_callback = function
    def on_erase_menu_item_activated(self, menu_item):
        self.erase_callback()
        return True

class List(gtk.ScrolledWindow):
    def __init__(self):
        gtk.ScrolledWindow.__init__(self)
        self.set_shadow_type(gtk.SHADOW_IN)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        self.tree_view = gtk.TreeView(self.list_store)
        self.tree_view.set_enable_search(False)
        self.add(self.tree_view)
        self.selection = self.tree_view.get_selection()
        self.selection.set_mode("single")
        self.erase_menu = EraseMenu()
        self.erase_menu.set_erase_callback(self.erase)
    def append(self, rows):
        n = 0
        for row in rows:
            i = self.list_store.append()
            self.list_store.set_value(i, 0, n)
            if type(row) is str:
                self.list_store.set_value(i, 1, row)
            else:
                m = 1
                for col in row:
                    self.list_store.set_value(i, m, col)
                    m += 1
            n += 1
    def update_selection(self, row):
        list_store, i = self.selection.get_selected()
        if type(row) is str:
            self.list_store.set_value(i, 1, row)
        else:
            m = 1
            for col in row:
                self.list_store.set_value(i, m, col)
                m += 1
    def clear(self):
        self.list_store.clear()
    def set_select_callback(self, function):
        self.selection.connect("changed", self.on_selection_changed)
        self.select_callback = function
    def set_activate_callback(self, function):
        self.tree_view.connect(
            "row-activated", self.on_tree_view_row_activated)
        self.activate_callback = function
    def set_erase_callback(self, function):
        self.tree_view.connect(
            "button-press-event", self.on_tree_view_button_press)
        self.tree_view.connect(
            "key-press-event", self.on_tree_view_key_press)
        self.erase_callback = function
    def on_selection_changed(self, selection):
        list_store, i = self.selection.get_selected()
        if i == None:
            n = None
        else:
            n = list_store.get_value(i, 0)
        self.select_callback(n)
        return True
    def on_tree_view_row_activated(self, tree_view, path, view_column):
        list_store, i = self.selection.get_selected()
        if i is not None:
            self.activate_callback(list_store.get_value(i, 0))
        return True
    def on_tree_view_button_press(self, tree_view, event):
        if event.button != 3:
            return False
        path = self.tree_view.get_path_at_pos(int(event.x), int(event.y))
        if path != None:
            self.tree_view.set_cursor_on_cell(path[0][0])
            self.erase_menu.popup(None, None, None, event.button, event.time)
        return True
    def popup_position(self, erase_menu):
        x, y = self.window.get_position()
        geometry1 = self.tree_view.window.get_geometry()
        self.erase_menu.realize()
        geometry2 = self.erase_menu.window.get_geometry()
        x += geometry1[0] + (geometry1[2] - geometry2[2]) / 2
        y += geometry1[1] + geometry1[3]
        return (x, y, 0)
    def on_tree_view_key_press(self, tree_view, event):
        if gtk.gdk.keyval_name(event.keyval) != "Delete":
            return False
        if self.selection.get_selected()[1] != None:
            self.erase_menu.popup(
                None, None, self.popup_position, 0, event.time)
        return True
    def erase(self):
        list_store, i = self.selection.get_selected()
        self.erase_callback(list_store.get_value(i, 0))

class OneColumnList(List):
    def __init__(self, label1):
        self.list_store = gtk.ListStore(
            gobject.TYPE_INT, gobject.TYPE_STRING)
        List.__init__(self)
        cell_renderer = gtk.CellRendererText()
        column1 = gtk.TreeViewColumn(label1, cell_renderer, text=1)
        column1.set_sort_column_id(1)
        column1.set_resizable(True)
        column1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree_view.append_column(column1)

class SessionList(List):
    def __init__(self, label1, label2, label3):
        # Second column in self.list_store is used to sort column
        # three.
        self.list_store = gtk.ListStore(
            gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING,
            gobject.TYPE_STRING, gobject.TYPE_STRING)
        List.__init__(self)
        cell_renderer = gtk.CellRendererText()
        column1 = gtk.TreeViewColumn(label1, cell_renderer, text=2)
        column1.set_sort_column_id(1)
        column1.set_resizable(True)
        column1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree_view.append_column(column1)
        column2 = gtk.TreeViewColumn(label2, cell_renderer, text=3)
        column2.set_sort_column_id(3)
        column2.set_resizable(True)
        column2.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree_view.append_column(column2)
        column3 = gtk.TreeViewColumn(label3, cell_renderer, text=4)
        column3.set_sort_column_id(4)
        column3.set_resizable(True)
        column3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree_view.append_column(column3)

class ListWindow(BasicWindow):
    def __init__(self):
        BasicWindow.__init__(self)
    def append_to_list(self, rows):
        self.list.append(rows)
    def update_list_selection(self, row):
        self.list.update_selection(row)
    def clear_list(self):
        self.list.clear()
    def set_add_callback(self, function):
        self.add_button.connect("clicked", self.on_add_button_clicked)
        self.add_callback = function
    def set_select_callback(self, function):
        self.select_callback = function
    def set_activate_callback(self, function):
        self.activate_callback = function
    def set_erase_callback(self, function):
        self.erase_callback = function
    def on_add_button_clicked(self, button):
        self.add_callback()
        return True
    def on_list_select(self, n):
        self.select_callback(n)
    def on_list_activate(self, n):
        self.activate_callback(n)
        return True
    def on_list_erase(self, n):
        self.erase_callback(n)
        return True
    
class DoubleListWindow(BasicWindow):
    def __init__(self):
        BasicWindow.__init__(self)
    def append_to_list1(self, rows):
        self.list1.append(rows)
    def append_to_list2(self, rows):
        self.list2.append(rows)
    def update_list_selection1(self, row):
        self.list1.update_selection(row)
    def update_list_selection2(self, row):
        self.list2.update_selection(row)
    def clear_list1(self):
        self.list1.clear()
    def clear_list2(self):
        self.list2.clear()
    def set_add_callback1(self, function):
        self.add_button1.connect("clicked", self.on_add_button_clicked1)
        self.add_callback1 = function
    def set_add_callback2(self, function):
        self.add_button2.connect("clicked", self.on_add_button_clicked2)
        self.add_callback2 = function
    def set_select_callback1(self, function):
        self.select_callback1 = function
    def set_select_callback2(self, function):
        self.select_callback2 = function
    def set_activate_callback1(self, function):
        self.activate_callback1 = function
    def set_activate_callback2(self, function):
        self.activate_callback2 = function
    def set_erase_callback1(self, function):
        self.erase_callback1 = function
    def set_erase_callback2(self, function):
        self.erase_callback2 = function
    def on_add_button_clicked1(self, button):
        self.add_callback1()
        return True
    def on_add_button_clicked2(self, button):
        self.add_callback2()
        return True
    def on_list_select1(self, n):
        self.select_callback1(n)
    def on_list_select2(self, n):
        self.select_callback2(n)
    def on_list_activate1(self, n):
        self.activate_callback1(n)
        return True
    def on_list_activate2(self, n):
        self.activate_callback2(n)
        return True
    def on_list_erase1(self, n):
        self.erase_callback1(n)
        return True
    def on_list_erase2(self, n):
        self.erase_callback2(n)
        return True

class TaskListWindow(DoubleListWindow):
    def __init__(self, conf):
        DoubleListWindow.__init__(self)
        self.window.set_title(_("Tasks"))
        self.window.set_modal(True)
        try:
            width = int(conf["task_list_window_width"])
            height = int(conf["task_list_window_height"])
            self.window.set_default_size(width, height)
        except:
            self.window.set_default_size(480, 240)
        self.hpaned = gtk.HPaned()
        self.vbox.pack_start(self.hpaned)
        try:
            pos = int(conf["task_list_window_divider"])
            self.hpaned.set_position(pos)
        except:
            pass
        vbox1 = gtk.VBox()
        vbox1.set_spacing(10)
        self.hpaned.pack1(vbox1, shrink = False)
        vbox2 = gtk.VBox()
        vbox2.set_spacing(10)
        self.hpaned.pack2(vbox2, shrink = False)
        self.add_button1 = ImageLabelButton(gtk.STOCK_ADD, _("New task"))
        vbox1.pack_start(self.add_button1, False, False)
        self.list1 = OneColumnList(_("Task"))
        self.list1.set_select_callback(self.on_list_select1)
        self.list1.set_activate_callback(self.on_list_activate1)
        self.list1.set_erase_callback(self.on_list_erase1)
        vbox1.pack_start(self.list1)
        self.add_button2 = ImageLabelButton(
            gtk.STOCK_ADD, _("New work session"))
        vbox2.pack_start(self.add_button2, False, False)
        self.list2 = SessionList(_("Date"), _("Time"), _("Detail"))
        self.list2.set_activate_callback(self.on_list_activate2)
        self.list2.set_erase_callback(self.on_list_erase2)
        vbox2.pack_start(self.list2)
        self.report_button = ImageLabelButton(
            gtk.STOCK_COPY, _("Create report"))
        self.vbox.pack_start(self.report_button, False, False)
        self.tooltips.set_tip(self.add_button1, _("Create a new task"))
        self.tooltips.set_tip(
            self.add_button2,
            _("Start a new work session of the selected task"))
        self.tooltips.set_tip(
            self.report_button, _("Generate a report of the selected task"))
        self.vbox.show_all()
    def set_report_callback(self, function):
        self.report_button.connect("clicked", self.on_report_button_clicked)
        self.report_callback = function
    def on_report_button_clicked(self, report_button):
        self.report_callback()
        return True
    def save_conf(self, conf):
        width, height = self.window.get_size()
        conf["task_list_window_width"] = str(width)
        conf["task_list_window_height"] = str(height)
        conf["task_list_window_divider"] = str(self.hpaned.get_position())

class UserEditWindow(BasicWindow):
    def __init__(self, parent, new = False):
        BasicWindow.__init__(self)
        self.window.set_modal(True)
        self.window.set_resizable(False)
        self.window.set_transient_for(parent.window)
        table = gtk.Table(2, 2)
        self.vbox.pack_start(table, False, False)
        table.set_row_spacings(5)
        table.set_col_spacings(5)
        table.attach(LeftLabel(_("User") + ":"), 0, 1, 0, 1)
        self.user_entry = Entry()
        self.user_entry.set_max_length(user_name_len)
        self.user_entry.set_activates_default(True)
        table.attach(self.user_entry, 1, 2, 0, 1)
        table.attach(LeftLabel(_("Password") + ":"), 0, 1, 1, 2)
        self.password_entry = Entry()
        self.password_entry.set_activates_default(True)
        self.password_entry.set_visibility(False)
        table.attach(self.password_entry, 1, 2, 1, 2)
        self.save_button = ImageLabelButton(gtk.STOCK_OK, _("Save"))
        self.vbox.pack_start(self.save_button, False, False)
        self.save_button.set_flags(gtk.CAN_DEFAULT)
        self.save_button.grab_default()
        if new:
            self.window.set_title(_("New user"))
            self.tooltips.set_tip(self.user_entry, _("New user name"))
            self.tooltips.set_tip(self.password_entry, _("New user password"))
            self.tooltips.set_tip(self.save_button, _("Save new user"))
        else:
            self.window.set_title(_("Edit user"))
            self.user_entry.set_editable(False)
            self.tooltips.set_tip(
                self.user_entry, _("Name of the user that is being edited"))
            self.tooltips.set_tip(
                self.password_entry, _("New password for user"))
            self.tooltips.set_tip(self.save_button, _("Save changes to user"))
        self.vbox.show_all()
    def show(self, user = None):
        if user is None:
            self.user_entry.grab_focus()
        else:
            self.user_entry.set_text(user)
            self.password_entry.grab_focus()
        BasicWindow.show(self)
    def hide(self):
        self.user_entry.set_text("")
        self.password_entry.set_text("")
        BasicWindow.hide(self)
    def set_save_callback(self, function):
        self.save_button.connect("clicked", self.on_save_button_clicked)
        self.save_callback = function
    def on_save_button_clicked(self, widget):
        data = {"user": self.user_entry.get_text(),
                "password": self.password_entry.get_text()}
        self.save_callback(data)
        return True

class UserListWindow(ListWindow):
    def __init__(self, conf):
        ListWindow.__init__(self)
        self.window.set_title(_("Users"))
        try:
            width = int(conf["user_list_window_width"])
            height = int(conf["user_list_window_height"])
            self.window.set_default_size(width, height)
        except:
            self.window.set_default_size(320, 200)
        self.add_button = ImageLabelButton(gtk.STOCK_ADD, _("Add user"))
        self.vbox.pack_start(self.add_button, False, False)
        self.list = OneColumnList(_("User"))
        self.list.set_activate_callback(self.on_list_activate)
        self.list.set_erase_callback(self.on_list_erase)
        self.vbox.pack_start(self.list)
        self.tooltips.set_tip(self.add_button, _("Add a new user"))
        self.vbox.show_all()
    def save_conf(self, conf):
        width, height = self.window.get_size()
        conf["user_list_window_width"] = str(width)
        conf["user_list_window_height"] = str(height)

class ConfigWindow(BasicWindow):
    def __init__(self, conf, image_path, icon16_path, icon32_path,
                 icon48_path):
        try: 
            icon16 = gtk.gdk.pixbuf_new_from_file(get_image_path(icon16_path))
            icon32 = gtk.gdk.pixbuf_new_from_file(get_image_path(icon32_path))
            icon48 = gtk.gdk.pixbuf_new_from_file(get_image_path(icon48_path))
            gtk.window_set_default_icon_list(icon16, icon32, icon48)
        except gobject.GError:
            pass
        BasicWindow.__init__(self)
        self.window.set_title(_("Configuration"))
        self.window.set_resizable(False)
        image = gtk.Image()
        image_path = get_image_path(image_path)
        image.set_from_file(image_path)
        self.vbox.pack_start(image, False, False)
        table = gtk.Table(2, 2)
        self.vbox.pack_start(table, False, False)
        table.set_row_spacings(5)
        table.set_col_spacings(5)
        table.attach(LeftLabel(_("User") + ":"), 0, 1, 0, 1)
        self.user_entry = Entry()
        self.user_entry.set_activates_default(True)
        table.attach(self.user_entry, 1, 2, 0, 1)
        table.attach(LeftLabel(_("Password") + ":"), 0, 1, 1, 2)
        self.password_entry = gtk.Entry()
        self.password_entry.set_activates_default(True)
        self.password_entry.set_visibility(False)
        table.attach(self.password_entry, 1, 2, 1, 2)
        self.connect_button = ImageLabelButton(gtk.STOCK_OK, _("Connect"))
        self.vbox.pack_start(self.connect_button, False, False)
        self.connect_button.set_flags(gtk.CAN_DEFAULT)
        self.connect_button.grab_default()
        self.tooltips.set_tip(
            self.user_entry, _("Database user") + "\n" + _("user") \
            + "[@" + _("server") + "[:" + _("port") + "]" + "]")
        self.tooltips.set_tip(self.password_entry, _("User password"))
        self.tooltips.set_tip(self.connect_button, _("Connect with database"))
        try:
            self.user_entry.set_text(conf['login'])
            self.password_entry.grab_focus()
        except KeyError:
            pass
        self.vbox.show_all()
    def set_connect_callback(self, function):
        self.connect_button.connect("clicked", self.on_connect_button_clicked)
        self.connect_callback = function
    def on_connect_button_clicked(self, widget):
        data = {}
        user_entry = self.user_entry.get_text()
        try:
            data["user"], data["host"] = user_entry.split("@", 1)
        except:
            data["user"] = user_entry
            data["host"] = "localhost"
        try:
            host, port = data["host"].split(":", 1)
            data["port"] = int(port)
            data["host"] = host
        except:
            data["port"] = 3306
        data["password"] = self.password_entry.get_text()
        self.connect_callback(data)
    def save_conf(self, conf):
        conf['login'] = self.user_entry.get_text()

class UserConfigWindow(ConfigWindow):
    def __init__(self, conf):
        ConfigWindow.__init__(
            self, conf, "tst10_splash.png", "tst10_16x16.xpm",
            "tst10_32x32.xpm", "tst10_48x48.xpm")

class AdminConfigWindow(ConfigWindow):
    def __init__(self, conf):
        ConfigWindow.__init__(
            self, conf, "tst10-admin_splash.png", "tst10-admin_16x16.xpm",
            "tst10-admin_32x32.xpm", "tst10-admin_48x48.xpm")
