# TST - work time tracker
# tst10/main.py: main routines
# Copyright (C) 2005 The TST Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

import os
import sys
import datetime
import locale
import gettext

from tst10 import inst_prefix

prefix = __name__.split(".")[0]

# Fix to use default locale on Windows.
if sys.platform[:3] == "win" and not os.environ.has_key("LANG"):
    lang = locale.getdefaultlocale()[0]
    if lang is not None:
        os.environ["LANG"] = lang
gettext.install(prefix, os.path.join(inst_prefix, "share", "locale"))

from tst10 import db
from tst10 import gui

def sec_to_str(ss):
    mm = ss / 60
    ss %= 60
    hh = mm / 60
    mm %= 60
    return "%02d:%02d:%02d" % (hh, mm, ss)

def utf8_cut(utf8_str, min, max):
    y = 0
    s1 = ""
    s2 = ""
    try:
        for x in range(min, max):
            c = utf8_str[x + y]
            s1 += c
            # If it is bigger than 127 then it is an special
            # character, so next one is taken too.
            if ord(c) > 127:
                y += 1
                s1 += utf8_str[x + y]
        s2 = utf8_str[max + y:]
    except IndexError:
        pass
    return s1, s2

def fmt(text):
    col = 45
    fmted = ""
    for line in text.splitlines():
        while True:
            tmp, line = utf8_cut(line, 0, col)
            if line[:1].isspace():
                line = line.lstrip()
            elif len(line) > 0 and len(tmp) >= col and not tmp[-1:].isspace():
                tmp = tmp.split(" ")
                if len(tmp) > 1:
                    line = tmp[-1] + line
                    tmp = tmp[:-1]
                tmp = " ".join(tmp)
            fmted += "|\t" + tmp + "\n"
            if len(line) == 0:
                break
    return fmted.rstrip()

class ConfReadError(Exception):
    pass

class ConfWriteError(Exception):
    pass

def conf_read(path):
    conf = {}
    try:
        stream = file(path)
        conf["login"] = stream.readline().strip()
        for line in stream.readlines():
            try:
                var, val = line.split(":", 1)
                conf[var.strip()] = val.strip()
            except ValueError:
                pass
        stream.close()
    except IOError, e:
        if e.errno != 2:
            msg = _("Error reading configuration") \
                  + ": %s" % os.strerror(e.errno)
            raise ConfReadError, msg
    return conf


def conf_write(path, conf):
    try:
        stream = file(path, "w")
        stream.write("%s\n" % conf["login"])
        keys = conf.keys()
        keys.sort()
        for var in keys:
            if var != "login":
                stream.write("%s: %s\n" % (var, conf[var]))
        stream.close()
    except IOError, e:
        msg = _("Error writing configuration") \
              + ": %s" % os.strerror(e.errno)
        raise ConfWriteError, msg

class UserMain:
    def __init__(self):
        self.task = None
        self.tasks = []
        self.session = None
        self.sessions = []
        self.conf_path = os.path.join(
            os.path.expanduser("~"), ".%src" % prefix)
        try:
            self.conf = conf_read(self.conf_path)
            error = False
        except ConfReadError, e:
            self.conf = {}
            msg = str(e)
            error = True
        self.ucw = gui.UserConfigWindow(self.conf)
        self.ucw.set_close_callback(self.on_ucw_close)
        self.ucw.set_connect_callback(self.on_ucw_connect)
        self.tlw = gui.TaskListWindow(self.conf)
        self.tlw.set_close_callback(self.on_tlw_close)
        self.tlw.set_add_callback1(self.on_tlw_add1)
        self.tlw.set_select_callback1(self.on_tlw_select1)
        self.tlw.set_activate_callback1(self.on_tlw_activate1)
        self.tlw.set_erase_callback1(self.on_tlw_erase1)
        self.tlw.set_add_callback2(self.on_tlw_add2)
        self.tlw.set_activate_callback2(self.on_tlw_activate2)
        self.tlw.set_erase_callback2(self.on_tlw_erase2)
        self.tlw.set_report_callback(self.on_tlw_report)
        self.tnw = gui.TaskEditWindow(self.conf, self.tlw, True)
        self.tnw.set_close_callback(self.on_tnw_close)
        self.tnw.set_save_callback(self.on_tnw_save)
        self.tew = gui.TaskEditWindow(self.conf, self.tlw)
        self.tew.set_close_callback(self.on_tew_close)
        self.tew.set_save_callback(self.on_tew_save)
        self.ww = gui.WorkWindow(self.conf)
        self.ww.set_stop_callback(self.on_ww_stop)
        self.sew = gui.SessionEditWindow(self.conf, self.tlw)
        self.sew.set_close_callback(self.on_sew_close)
        self.sew.set_save_callback(self.on_sew_save)
        self.rw = gui.ReportWindow(self.conf, self.tlw)
        self.rw.set_close_callback(self.on_rw_close)
        self.rw.set_update_callback(self.on_rw_update)
        self.rw.set_copy_callback(self.on_rw_copy)
        self.db = db.UserDB()
        self.ucw.show()
        if error:
            self.ucw.set_message(msg)
        gui.begin()
    def on_ucw_close(self):
        self.ucw.hide()
        gui.end()
        sys.exit(0)
    def on_ucw_connect(self, data):
        self.ucw.set_working(True)
        try:
            self.db.connect(
                data["user"], data["password"], data["host"], data["port"])
            self.tlw.show()
            self.ucw.hide()
            self.tlw.set_working(True)
            try:
                self.tasks = self.db.tasks()
                self.update_tlw_list1()
            except db.Error, e:
                self.tlw.set_message(str(e))
            self.tlw.set_working(False)
        except db.Error, e:
            self.ucw.set_message(str(e))
        self.ucw.set_working(False)
    def update_tlw_list1(self):
        self.tlw.clear_list1()
        self.tlw.append_to_list1([task["name"] for task in self.tasks])
    def update_tlw_list2(self):
        tmp = []
        for session in self.sessions:
            d = session["date"]
            t = session["time"]
            detail = utf8_cut(session["detail"], 0, 80)[0].replace("\n", " ")
            if len(session["detail"]) != len(detail):
                detail += "..."
            # Fist element is used to sort the first column depending
            # on the date and the id of the session.
            tmp.append(
                ("%s%011d" % (str(d), session["id"]),
                 d.strftime("%x"), str(t), detail))
        self.tlw.clear_list2()
        self.tlw.append_to_list2(tmp)
        self.session = None
    def on_tlw_close(self):
        try:
            new_conf = self.conf.copy()
            self.ucw.save_conf(new_conf)
            self.tlw.save_conf(new_conf)
            self.tnw.save_conf(new_conf)
            self.tew.save_conf(new_conf)
            self.ww.save_conf(new_conf)
            self.sew.save_conf(new_conf)
            self.rw.save_conf(new_conf)
            if self.conf != new_conf:
                conf_write(self.conf_path, new_conf)
            self.tlw.hide()
            gui.end()
            sys.exit(0)
        except ConfWriteError, e:
            self.tlw.set_message(str(e))
    def on_tlw_select1(self, n):
        self.task = n
        if n is not None:
            self.tlw.set_working(True)
            try:
                self.sessions = self.db.sessions(self.tasks[n]["id"])
                self.update_tlw_list2()
                self.tlw.set_message()
            except db.Error, e:
                self.tlw.set_message(str(e))
            self.tlw.set_working(False)
        else:
            self.tlw.clear_list2()
            self.sessions = []
    def on_tlw_activate1(self, n):
        if self.task is not None:
            self.tew.show(self.tasks[self.task])
    def on_tlw_activate2(self, n):
        session = self.sessions[n]
        task = self.tasks[self.task]
        self.sew.show(task, session)
        self.tlw.set_message()
        self.session = n
    def on_tlw_erase1(self, n):
        self.tlw.set_working(True)
        try:
            self.db.del_task(self.tasks[n]["id"])
            self.tasks = self.db.tasks()
            self.update_tlw_list1()
            self.tlw.set_message()
        except db.Error, e:
            self.tlw.set_message(str(e))
        self.tlw.set_working(False)
    def on_tlw_erase2(self, n):
        self.tlw.set_working(True)
        try:
            self.db.del_session(self.sessions[n]["id"])
            self.sessions = self.db.sessions(self.tasks[self.task]["id"])
            self.update_tlw_list2()
            self.tlw.set_message()
        except db.Error, e:
            self.tlw.set_message(str(e))
        self.tlw.set_working(False)
    def on_tlw_add1(self):
        self.tnw.show()
        self.tlw.set_message()
    def tlw_task_is_selected(self):
        if self.task is None:
            self.tlw.set_message(_("No task is selected") + ".")
            return False
        return True
    def on_tlw_add2(self):
        if self.tlw_task_is_selected():
            self.ww.start(
                self.tasks[self.task]["name"],
                fmt(self.tasks[self.task]["notes"]))
            self.tlw.hide()
            self.tlw.set_message()
    def on_tlw_report(self):
        if self.tlw_task_is_selected():
            if len(self.sessions) == 0:
                filter = (datetime.date.today(),) * 2
            else:
                filter = (
                    self.sessions[0]["date"], self.sessions[-1]["date"])
            self.tlw.set_working(True)
            self.rw.show(filter)
            self.tlw.set_working(False)
            self.tlw.set_message()
    def on_tnw_close(self):
        self.tnw.hide()
    def on_tnw_save(self, data):
        self.tnw.set_working(True)
        try:
            self.db.add_task(data["name"], data["notes"])
            self.tnw.hide()
            self.tlw.set_working(True)
            try:
                self.tasks = self.db.tasks()
                self.update_tlw_list1()
            except db.Error, e:
                self.tlw.set_message(str(e))
            self.tlw.set_working(False)
        except db.Error, e:
            self.tnw.set_message(str(e))
        self.tnw.set_working(False)
    def on_tew_close(self):
        self.tew.hide()
    def on_tew_save(self, data):
        task_id = self.tasks[self.task]["id"]
        self.tew.set_working(True)
        try:
            self.db.mod_task(task_id, data["name"], data["notes"])
            self.tasks[self.task]["name"] = data["name"]
            self.tasks[self.task]["notes"] = data["notes"]
            self.tlw.update_list_selection1(data["name"])
            self.tew.hide()
        except db.Error, e:
            self.tew.set_message(str(e))
        self.tew.set_working(False)
    def on_ww_stop(self, t):
        task = self.tasks[self.task]
        session = {"detail": "", "date": datetime.date.today(), "time": t}
        self.tlw.show()
        self.sew.show(task, session, True)
    def on_sew_close(self):
        self.session = None
        self.sew.hide()
    def on_sew_save(self, data):
        task_id = self.tasks[self.task]["id"]
        self.sew.set_working(True)
        try:
            if self.session is None:
                self.db.add_session(
                    task_id, data["detail"], data["date"], data["time"])
            else:
                session_id = self.sessions[self.session]["id"]
                self.db.mod_session(
                    session_id, data["detail"], data["date"], data["time"])
            self.sew.hide()
            self.tlw.set_working(True)
            try:
                self.sessions = self.db.sessions(task_id)
                self.update_tlw_list2()
            except db.Error, e:
                self.tlw.set_message(str(e))
            self.tlw.set_working(False)
        except db.Error, e:
            self.sew.set_message(str(e))
        self.sew.set_working(False)
    def on_rw_close(self):
        self.rw.hide()
    def on_rw_update(self, filter):
        if filter[0] > filter[1]:
            filter = (filter[1], filter[0])
        report = ""
        self.rw.set_working(True)
        try:
            self.report_sessions = self.db.sessions(
                self.tasks[self.task]["id"], filter[0], filter[1])
            self.report_sum = self.db.sum_sessions(
                self.tasks[self.task]["id"], filter[0], filter[1])
            report = "%s\n" % self.tasks[self.task]["name"]
            report += (_("From") + " %s " + _("to") + " %s\n\n") \
                  % (filter[0].strftime("%x"), filter[1].strftime("%x"))
            for session in self.report_sessions:
                report += "%s %s\n" \
                          % (session["date"].strftime("%x"),
                             str(session["time"]))
                detail = fmt(session["detail"])
                if len(detail) > 0:
                    report += "%s\n" % detail
                report += "\n"
            report += (_("Total") + ": %s\n") % sec_to_str(self.report_sum)
            self.rw.set_message()
        except db.Error, e:
            self.rw.set_message(str(e))
        self.rw.set_working(False)
        self.rw.update(report)
    def on_rw_copy(self):
        self.rw.hide()

class AdminMain:
    def __init__(self):
        self.conf_path = os.path.join(
            os.path.expanduser("~"), ".%s-adminrc" % prefix)
        try:
            self.conf = conf_read(self.conf_path)
            error = False
        except ConfReadError, e:
            self.conf = {}
            msg = str(e)
            error = True
        self.acw = gui.AdminConfigWindow(self.conf)
        self.acw.set_close_callback(self.on_acw_close)
        self.acw.set_connect_callback(self.on_acw_connect)
        self.ulw = gui.UserListWindow(self.conf)
        self.ulw.set_close_callback(self.on_ulw_close)
        self.ulw.set_add_callback(self.on_ulw_add)
        self.ulw.set_activate_callback(self.on_ulw_activate)
        self.ulw.set_erase_callback(self.on_ulw_erase)
        self.unw = gui.UserEditWindow(self.ulw, True)
        self.unw.set_close_callback(self.on_unw_close)
        self.unw.set_save_callback(self.on_unw_save)
        self.uew = gui.UserEditWindow(self.ulw)
        self.uew.set_close_callback(self.on_uew_close)
        self.uew.set_save_callback(self.on_uew_save)
        self.db = db.AdminDB()
        self.acw.show()
        if error:
            self.acw.set_message(msg)
        gui.begin()
    def on_acw_close(self):
        self.acw.hide()
        gui.end()
        sys.exit(0)
    def on_acw_connect(self, data):
        self.acw.set_working(True)
        try:
            self.db.connect(
                data["user"], data["password"], data["host"], data["port"])
            self.ulw.show()
            self.acw.hide()
            self.ulw.set_working(True)
            try:
                self.users = self.db.users()
                self.ulw.clear_list()
                self.ulw.append_to_list(self.users)
            except db.Error, e:
                self.ulw.set_message(str(e))
            self.ulw.set_working(False)
        except db.Error, e:
            self.acw.set_message(str(e))
        self.acw.set_working(False)
    def on_ulw_close(self):
        try:
            new_conf = self.conf.copy()
            self.acw.save_conf(new_conf)
            self.ulw.save_conf(new_conf)
            if self.conf != new_conf:
                conf_write(self.conf_path, new_conf)
            self.ulw.hide()
            gui.end()
            sys.exit(0)
        except ConfWriteError, e:
            self.ulw.set_message(str(e))
    def on_ulw_add(self):
        self.unw.show()
        self.ulw.set_message()
    def on_ulw_activate(self, n):
        self.uew.show(self.users[n])
        self.ulw.set_message()
    def on_ulw_erase(self, n):
        self.ulw.set_working(True)
        try:
            self.db.del_user(self.users[n])
            self.users = self.db.users()
            self.ulw.clear_list()
            self.ulw.append_to_list(self.users)
            self.ulw.set_message()
        except db.Error, e:
            self.ulw.set_message(str(e))
        self.ulw.set_working(False)
    def on_unw_close(self):
        self.unw.hide()
    def on_unw_save(self, data):
        self.unw.set_working(True)
        try:
            self.db.add_user(data["user"], data["password"])
            self.unw.hide()
            self.ulw.set_working(True)
            try:
                self.users = self.db.users()
                self.ulw.clear_list()
                self.ulw.append_to_list(self.users)
            except db.Error, e:
                self.ulw.set_message(str(e))
            self.ulw.set_working(False)
        except db.Error, e:
            self.unw.set_message(str(e))
        self.unw.set_working(False)
    def on_uew_close(self):
        self.uew.hide()
    def on_uew_save(self, data):
        self.uew.set_working(True)
        try:
            self.db.set_user_password(data["user"], data["password"])
            self.uew.hide()
        except db.Error, e:
            self.uew.set_message(str(e))
        self.uew.set_working(False)
